"""
    Extend the standard ConfigParser.
"""

import ConfigParser
import re
import StringIO
import os

SECTION_HEADER = re.compile( r'^\[([A-Za-z][A-Za-z0-9_]*)\]', re.MULTILINE )

class ConfigParserExt( ConfigParser.ConfigParser ):
    """
        Allow ordered query of sections in a config file;  some sections
        may be "privileged", and not show up in the main list.
    """

    def __init__( self, special_sections=( 'DEFAULT', ) ):

        ConfigParser.ConfigParser.__init__( self )
        self.clear()
        self._special_sections = list( special_sections )

    #
    #   Accessors
    #
    def listSpecialSections( self, include_all=1 ):

        if include_all:
            return tuple( self._special_sections )
        else:
            return filter( lambda x, sections=self.sections(): x in sections
                         , self._special_sections )

    def listOtherSections( self ):

        return tuple( self._other_sections )

    #
    #   Mutators
    #
    def clear( self ):
        """
        """
        self._defaults          = {}
        self._special_sections  = []
        self._other_sections    = []

    def setDefaults( self, **kw ):
        """
            Add kw to our defaults
        """
        self._defaults.update( kw )

    def read( self, filename ):
        """
        """
        if filename is None:
            raise ValueError, "Must supply 'filename'."

        if '%' in filename:
            filename = filename % self._defaults

        dir, fn = os.path.split( filename )
        self._defaults[ 'config_dir' ] = dir
        self._defaults[ 'config_file' ] = fn

        file = open( filename )

        contents = file.read()

        self._other_sections.extend(
                filter( lambda x, ss=self.listSpecialSections():
                                    x not in ss
                      , SECTION_HEADER.findall( contents ) ) )

        self.defaults().update( self._defaults )
        ConfigParser.ConfigParser.read( self, filename )
